<?php

/**
 * The main class of the plugin.
 * All the magic happens here.
 */
class PhotoSwipePlugin {

	/**
	 * Class-wide flag that the PhotoSwipe overlay UI has already been initialized
	 *
	 * @var boolean
	 */
	private static $photoswipeUiIsInitialized = false;

	/**
	 * Class-wide index counter for the shown images
	 *
	 * @var int
	 */
	private static $lastusedDataIndex = 0;

	/**
	 * Callback function called for [photoswipeimage] tags which returns the HTML code for a single PhotoSwipe image.
	 *
	 * @param unknown $action
	 * @param array $attr
	 *        	the attributes given in the tag
	 * @param unknown $content
	 * @param unknown $params
	 * @param unknown $node_object
	 * @return boolean|string the HTML code for a single PhotoSwipe image
	 */
	static function getImageHtml($action, $attr, $content, $params, $node_object) {
		if ($action == 'validate') {
			// not used for now
			return true;
		}
		// the name of the image - and its relative path
		$img = $attr ['default'];
		$imgPathRel = 'fp-content/' . $img;
		$imgUrl = BLOG_BASEURL . 'fp-content/' . $img;
		if (!file_exists($imgPathRel)) {
			return 'Given image file does not exist.';
		}

		// image title will be its file name - or the title from the tag attributes, if given
		$title = basename($imgPathRel);
		if (isset($attr ['title'])) {
			$title = wp_specialchars($attr ['title']);
		}

		// image may float, according the the given float attribute
		$float = ' class="nofloat" ';
		if (isset($attr ['float'])) {
			$float = ' class="float' . $attr ['float'] . '"';
		}

		// to get the HTML code for preview image, we use the Flatpress standard function do_bbcode_img()
		$previewHtml = do_bbcode_img(null, $attr, null, null, null);
		// but: we don't need the popup link surrounding the resulting <img> tag
		preg_match('/<img[^>]*/u', $previewHtml, $matches);
		$previewHtml = $matches [0];
		if ($previewHtml [strlen($previewHtml) - 1] === '/') {
			$previewHtml = substr($previewHtml, 0, strlen($previewHtml) - 1);
		}
		// add some additional attributes to the <img> tag and close it properly
		$previewHtml .= ' class="thumbnail" itemprop="thumbnail" title="' . $title . '">';

		// PhotoSwipe needs to know the dimensions of the image - so we read them
		$imgsize = getimagesize($imgPathRel);

		// know lets assemble the whole HTML code - including the overlay HTML, if not inserted into the DOM before
		$imgHtml = self::getPhotoSwipeOverlay() . //
		'<div class="photoswipe">
				<figure ' . //
		'itemprop="associatedMedia" ' . //
		'itemscope ' . //
		'itemtype="http://schema.org/ImageObject" ' . //
		'data-index="' . self::$lastusedDataIndex . '" ' . //
		$float . ' ' . //
		'>' . //
		'<a ' . //
		'href="' . $imgUrl . '" ' . //
		'itemprop="contentUrl" ' . //
		'data-size="' . $imgsize [0] . 'x' . $imgsize [1] . '" ' . //
		'data-index="' . self::$lastusedDataIndex . '" ' . //
		'title="' . $title . '" ' . //
		'>' . //
		$previewHtml . //
		'</a>' . //
		'<figcaption>' . $title . '</figcaption>' . //
		'</figure>' . //
		'</div>';

		self::$lastusedDataIndex++;
		return $imgHtml;
	}

	// getImageHtml()

	/**
	 * Callback function called for [photoswipegallery] tags which returns the HTML code for a PhotoSwipe gallery.
	 *
	 * @param unknown $action
	 * @param array $attr
	 *        	the attributes given in the tag
	 * @param unknown $content
	 * @param unknown $params
	 * @param unknown $node_object
	 * @return boolean|string the HTML code for a PhotoSwipe gallery
	 */
	static function getGalleryHtml($action, $attr, $content, $params, $node_object) {
		if ($action == 'validate') {
			// not used for now
			return true;
		}

		// gallery dir is set as tag attribute
		$dir = $attr ['default'];

		// sanitize first
		if (strpos($dir, '..') !== false) {
			return 'ERROR: Invalid gallery dir given';
		}
		// check if dir exists
		if (!file_exists("fp-content/" . $dir)) {
			return '<b>ERROR:</b> Directory ' . $dir . ' does not exist';
		}
		// force slash at the end
		if (substr($dir, -1) != '/') {
			$dir .= '/';
		}

		// read images from gallery directory (code taken from Quickgallery plugin)
		$d = substr_replace($dir, IMAGES_DIR, 0, 7);
		$fs = new fs_filelister($d);
		$l = $fs->getlist();
		sort($l);

		// read image caption from texte.conf (if existant in the gallery directory)
		$textsprop = new PhotoSwipeSimpleProperties();
		if (file_exists("fp-content/" . $dir . "texte.conf")) {
			$texts = file("fp-content/" . $dir . "texte.conf");
			foreach ($texts as $currentline) {
				if (strpos($currentline, " = ") < 3)
					continue;
				$currentline = trim($currentline);
				$image = substr($currentline, 0, strpos($currentline, " = "));
				$descript = substr($currentline, (strpos($currentline, " = ") + 3));
				$textsprop->set($image, htmlentities($descript, ENT_COMPAT | ENT_HTML401, "ISO-8859-1"));
			}
		}

		// call getImageHtml() to get the image's HTML code for each image in the gallery dir
		$imgattr = $attr;
		$str = '<div class="img-gallery ' . sanitize_title($dir) . '">';
		foreach ($l as $f) {
			// azett: set the image's caption as title attribut of the img tag
			if ($f == 'texte.conf') {
				continue;
			}
			$imgattr ['default'] = $dir . $f;
			$imgattr ['title'] = $textsprop->get($f);
			// /azett
			$str .= self::getImageHtml($action, $imgattr, $content, $params, $node_object);
			// $str .= do_bbcode_img($action, $imgattr, $content, $params, $node_object);
		}
		return $str . '</div>';
	}

	// getGalleryHtml()

	/**
	 * Returns the overlay HTML.
	 * This needs to be inserted into the DOM only once, so a second call will return an empty string.
	 *
	 * @return string the PhotoSwipe overlay HTML
	 */
	static function getPhotoSwipeOverlay() {
		$photoswipeoverlay = self::$photoswipeUiIsInitialized ? '' : '<div class="pswp" tabindex="-1" role="dialog" aria-hidden="true">' . //
		'<div class="pswp__bg"></div>' . //
		'<div class="pswp__scroll-wrap">' . //
		'<div class="pswp__container">' . //
		'<div class="pswp__item"></div>' . //
		'<div class="pswp__item"></div>' . //
		'<div class="pswp__item">' . //
		'</div>' . //
		'</div>' . //
		'<div class="pswp__ui pswp__ui--hidden">' . //
		'<div class="pswp__top-bar">' . //
		'<div class="pswp__counter"></div>' . //
		'<button class="pswp__button pswp__button--close" title="' . PHOTOWIPE_TEXT_CLOSEBUTTON . '"></button>' . //
		'<button class="pswp__button pswp__button--share" title="' . PHOTOWIPE_TEXT_SHAREBUTTON . '"></button>' . //
		'<button class="pswp__button pswp__button--fs" title="' . PHOTOWIPE_TEXT_FULLSCREENBUTTON . '"></button>' . //
		'<button class="pswp__button pswp__button--zoom" title="' . PHOTOWIPE_TEXT_ZOOMBUTTON . '"></button>' . //
		'<div class="pswp__preloader">' . //
		'<div class="pswp__preloader__icn">' . //
		'<div class="pswp__preloader__cut">' . //
		'<div class="pswp__preloader__donut"></div>' . //
		'</div>' . //
		'</div>' . //
		'</div>' . //
		'</div>' . //
		'<div class="pswp__share-modal pswp__share-modal--hidden pswp__single-tap">' . //
		'<div class="pswp__share-tooltip"></div>' . //
		'</div>' . //
		'<button class="pswp__button pswp__button--arrow--left" title="' . PHOTOWIPE_TEXT_PREVIMGBUTTON . '"></button>' . //
		'<button class="pswp__button pswp__button--arrow--right" title="' . PHOTOWIPE_TEXT_NEXTIMGBUTTON . '"></button>' . //
		'<div class="pswp__caption">' . //
		'<div class="pswp__caption__center"></div>' . //
		'</div>' . //
		'</div>' . //
		'</div>' . //
		'</div>';
		self::$photoswipeUiIsInitialized = true;
		return $photoswipeoverlay;
	}

	/**
	 * Header hook for loading the PhotoSwipe scripts.
	 * Echoes the <script> tags.
	 */
	static function echoScriptTags() {
		$pdir = plugin_geturl('photoswipe');
		echo '<!-- PhotoSwipe -->
';
		if (!function_exists('plugin_jquery_head')) {
			echo '<script type="text/javascript" src="' . $pdir . 'res/jquery-2.2.2/jquery-2.2.2.min.js"></script>
';
		}
	echo '
	<script type="text/javascript" src="' . $pdir . 'res/photoswipe-4.1.1/photoswipe-ui-default.min.js"></script>
	<script type="text/javascript" src="' . $pdir . 'res/photoswipe-4.1.1/photoswipe.min.js"></script>
	<script type="text/javascript" src="' . $pdir . 'res/photoswipe.js"></script>
	<link rel="stylesheet" property="stylesheet" href="' . $pdir . 'res/photoswipe-4.1.1/default-skin/default-skin.css">
	<link media="screen,projection,handheld" href="' . $pdir . 'res/photoswipe-4.1.1/photoswipe.css" type="text/css" rel="stylesheet">
	<!-- /PhotoSwipe -->';
	}

	/**
	 * Initializes the BBCode tags of the plugin.
	 */
	public static function initializePluginTags() {
		// get the global bbcode object
		$bbcode = plugin_bbcode_init();

		// add tag "photoswipegallery"
		$bbcode->addCode('photoswipegallery', // tag name: this will go between square brackets
		'callback_replace_single', // type of action: we'll use a callback function
		'PhotoSwipePlugin::getGalleryHtml', // name of the callback function
		array(
			'usecontent_param' => array(
				'default'
			)
		), // supported parameters: "default" is [acronym=valore]
		'inline', // type of the tag, inline or block, etc
		array(
			'listitem',
			'block',
			'inline',
			'link'
		), // type of elements in which you can use this tag
		array()); // type of elements where this tag CAN'T go (in this case, none, so it can go everywhere)
		$bbcode->setCodeFlag('photoswipegallery', 'closetag', BBCODE_CLOSETAG_FORBIDDEN); // a closing tag is forbidden (no [/tag])

		// add tag "photoswipeimage"
		$bbcode->addCode('photoswipeimage', // tag name: this will go between square brackets
		'callback_replace_single', // type of action: we'll use a callback function
		'PhotoSwipePlugin::getImageHtml', // name of the callback function
		array(
			'usecontent_param' => array(
				'default'
			)
		), // supported parameters: "default" is [acronym=valore]
		'inline', // type of the tag, inline or block, etc
		array(
			'listitem',
			'block',
			'inline',
			'link'
		), // type of elements in which you can use this tag
		array()); // type of elements where this tag CAN'T go (in this case, none, so it can go everywhere)
		$bbcode->setCodeFlag('photoswipeimage', 'closetag', BBCODE_CLOSETAG_FORBIDDEN); // a closing tag is forbidden (no [/tag])
	}

	// initializePluginTags()
} // class PhotoSwipePlugin